/*
 *   Copyright 2008 by Aaron Seigo <aseigo@kde.org>
 *   Copyright 2008 Marco Martin <notmart@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "svgpanel.h"

#include <QPainter>
#include <QSize>

namespace Plasma
{

SvgPanel::SvgPanel(const QString& imagePath, QObject* parent)
        : QObject(parent),
          bFlags(DrawTop|DrawBottom|DrawLeft|DrawRight),
          cachedBackground(0),
          validCache(false)
{
   background = new Svg(imagePath, this);
   background->setContentType(Svg::ImageSet);
   updateSizes();
   panelSize = background->size();
}

SvgPanel::~SvgPanel()
{
}

void SvgPanel::setFile(const QString& imagePath)
{
   if (imagePath != background->file()) {
       validCache = false;
   }

   delete background;
   background = new Svg(imagePath, this);
   //FIXME: background->setFile doesn't work...
   background->setFile(imagePath);
}

QString SvgPanel::file() const
{
   return background->file();
}

void SvgPanel::setBorderFlags(const BorderFlags flags)
{
   if (flags != bFlags) {
       validCache = false;
   }

   bFlags = flags;

   updateSizes();
}

SvgPanel::BorderFlags SvgPanel::borderFlags() const
{
   return bFlags;
}

void SvgPanel::resize(const QSizeF& size)
{
   if (size != panelSize) {
       validCache = false;
   }

   panelSize = size;
   contentWidth = panelSize.width() - leftWidth - rightWidth;
   contentHeight = panelSize.height() - topHeight  - bottomHeight;

   topOffset = 0 - topHeight;
   leftOffset = 0 - leftWidth;
   rightOffset = contentWidth;
   bottomOffset = contentHeight;
   contentTop = 0;
   contentLeft = 0;

   scaledSize = QSizeF(panelSize.width() -
                       (leftWidth+rightWidth) +
                       panelSize.width()*(((qreal)(leftWidth+rightWidth))/panelSize.width()),
                       panelSize.height() -
                       (topHeight+bottomHeight) +
                       panelSize.height()*(((qreal)(topHeight+bottomHeight))/panelSize.height()));
}

void SvgPanel::paint(QPainter* painter, const QPointF& point)
{
   if (!cachedBackground || !validCache) {

       delete cachedBackground;
       cachedBackground = new QPixmap(leftWidth + contentWidth + rightWidth, topHeight + contentHeight + bottomHeight);
       cachedBackground->fill(Qt::transparent);
       QPainter p(cachedBackground);
       p.translate(leftWidth, topHeight);
       p.setCompositionMode(QPainter::CompositionMode_Source);
       p.setRenderHint(QPainter::SmoothPixmapTransform);

       //FIXME: This is a hack to fix a drawing problems with svg files where a thin transparent border is drawn around the svg image.
       //       the transparent border around the svg seems to vary in size depending on the size of the svg and as a result increasing the
       //       svg image by 2 all around didn't resolve the issue. For now it resizes based on the border size.


       //CENTER
       background->resize(scaledSize.width(), scaledSize.height());
       background->paint(&p, QRect(contentLeft-leftWidth, contentTop-topHeight, contentWidth+leftWidth*2, contentHeight+topHeight*2), "center");
       background->resize();

       //EDGES
       if ((bFlags & DrawLeft) && (bFlags & DrawTop)) {
           background->paint(&p, QRect(leftOffset, topOffset, leftWidth, topHeight), "topleft");
       }
       if ((bFlags & DrawRight) && (bFlags & DrawTop)) {
           background->paint(&p, QRect(rightOffset, topOffset,rightWidth, topHeight), "topright");
       }
       if ((bFlags & DrawLeft) && (bFlags & DrawBottom)) {
           background->paint(&p, QRect(leftOffset, bottomOffset, leftWidth, bottomHeight), "bottomleft");
       }
       if ((bFlags & DrawRight) && (bFlags & DrawBottom)) {
           background->paint(&p, QRect(rightOffset, bottomOffset, rightWidth, bottomHeight), "bottomright");
       }

       //SIDES
       if (background->elementExists("hint-stretch-borders")) {
           if (bFlags & DrawLeft) {
               background->resize(background->size().width(), scaledSize.height());
               background->paint(&p, QRect(leftOffset, contentTop, leftWidth, contentHeight), "left");
               background->resize();
           }
           if (bFlags & DrawRight) {
               background->resize(background->size().width(), scaledSize.height());
               background->paint(&p, QRect(rightOffset, contentTop, rightWidth, contentHeight), "right");
               background->resize();
           }
           if (bFlags & DrawTop) {
               background->resize(scaledSize.width(), background->size().height());
               background->paint(&p, QRect(contentLeft, topOffset, contentWidth, topHeight), "top");
               background->resize();
           }
           if (bFlags & DrawBottom) {
               background->resize(scaledSize.width(), background->size().height());
               background->paint(&p, QRect(contentLeft, bottomOffset, contentWidth, bottomHeight), "bottom");
               background->resize();
           }
       } else {
           if (bFlags & DrawLeft) {
               QPixmap left(leftWidth, leftHeight);
               left.fill(Qt::transparent);

               QPainter sidePainter(&left);
               sidePainter.setCompositionMode(QPainter::CompositionMode_Source);
               background->paint(&sidePainter, QPoint(0, 0), "left");

               p.drawTiledPixmap(QRect(leftOffset, contentTop, leftWidth, contentHeight), left);
           }

           if (bFlags & DrawRight) {
               QPixmap right(rightWidth, leftHeight);
               right.fill(Qt::transparent);

               QPainter sidePainter(&right);
               sidePainter.setCompositionMode(QPainter::CompositionMode_Source);
               background->paint(&sidePainter, QPoint(0, 0), "right");

               p.drawTiledPixmap(QRect(rightOffset, contentTop, rightWidth, contentHeight), right);
           }

           if (bFlags & DrawTop) {
               QPixmap top(topWidth, topHeight);
               top.fill(Qt::transparent);

               QPainter sidePainter(&top);
               sidePainter.setCompositionMode(QPainter::CompositionMode_Source);
               background->paint(&sidePainter, QPoint(0, 0), "top");

               p.drawTiledPixmap(QRect(contentLeft, topOffset, contentWidth, topHeight), top);
           }

           if (bFlags & DrawBottom) {
               QPixmap bottom(topWidth, bottomHeight);
               bottom.fill(Qt::transparent);

               QPainter sidePainter(&bottom);
               sidePainter.setCompositionMode(QPainter::CompositionMode_Source);
               background->paint(&sidePainter, QPoint(0, 0), "bottom");

               p.drawTiledPixmap(QRect(contentLeft, bottomOffset, contentWidth, bottomHeight), bottom);
           }
       }

       // re-enable this once Qt's svg rendering is un-buggered
       //resize(contentWidth, contentHeight);
       //paint(&p, QRect(contentLeft, contentTop, contentWidth, contentHeight), "center");
   }

   //p2->drawPixmap(paintRect, *cachedBackground, paintRect.translated(-leftOffset,-topOffset));
   painter->drawPixmap(point, *cachedBackground);
}

void SvgPanel::updateSizes()
{
   if (bFlags & DrawTop) {
       topHeight = background->elementSize("top").height();
   } else {
       topHeight = 0;
   }
   if (bFlags & DrawLeft) {
       leftWidth =background->elementSize("left").width();
   } else {
       leftWidth = 0;
   }
   if (bFlags & DrawRight) {
       rightWidth = background->elementSize("right").width();
   } else {
       topHeight = 0;
   }
   if (bFlags & DrawBottom) {
       bottomHeight = background->elementSize("bottom").height();
   } else {
       bottomHeight = 0;
   }

   //since it's rectangular, topWidth and bottomWidth must be the same
   topWidth = background->elementSize("top").width();
   leftHeight = background->elementSize("left").height();
}


} // Plasma namespace

#include "svgpanel.moc"
